# -*- coding: utf-8 -*-
import datetime
from south.creator.freezer import freeze_apps
from south.db import db
from south.v2 import SchemaMigration

from django.conf import settings


ACTIVITYLOG_PROFILE_MODEL = getattr(settings, 'ACTIVITYLOG_PROFILE_MODEL',
    getattr(settings, 'AUTH_PROFILE_MODULE', 'auth.User'))
apm_key = ACTIVITYLOG_PROFILE_MODEL.lower()
apm_app = ACTIVITYLOG_PROFILE_MODEL.split('.')[0]
try:
    initial_migration = settings.ACTIVITYLOG_PROFILE_INITIAL_MIGRATION
except AttributeError:
    initial_migration = '0001_initial.py'
    if apm_key == 'auth.user':
        initial_migration = None
if initial_migration:
    depends_on = (
            (apm_app, initial_migration),
    )
else:
    depends_on = None


class Migration(SchemaMigration):

    depends_on = depends_on

    def forwards(self, orm):
        # Adding model 'UserAgent'
        db.create_table('activitylog_useragent', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('modified', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('cache_version', self.gf('django.db.models.fields.PositiveIntegerField')(default=0)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255, db_index=True)),
        ))
        db.send_create_signal('activitylog', ['UserAgent'])

        # Adding model 'IP'
        db.create_table('activitylog_ip', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('modified', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('cache_version', self.gf('django.db.models.fields.PositiveIntegerField')(default=0)),
            ('address', self.gf('django.db.models.fields.IPAddressField')(null=True, default=None, max_length=15, blank=True, unique=True, db_index=True)),
            ('number', self.gf('django.db.models.fields.BigIntegerField')(default=None, unique=True, null=True, blank=True)),
            ('hostname', self.gf('django.db.models.fields.CharField')(default=None, max_length=255, null=True, blank=True)),
        ))
        db.send_create_signal('activitylog', ['IP'])

        # Adding model 'Backlink'
        db.create_table('activitylog_backlink', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('modified', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('cache_version', self.gf('django.db.models.fields.PositiveIntegerField')(default=0)),
            ('site', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['sites.Site'])),
            ('url', self.gf('django.db.models.fields.URLField')(max_length=100)),
            ('referrer', self.gf('django.db.models.fields.URLField')(max_length=100)),
            ('visits', self.gf('django.db.models.fields.PositiveIntegerField')(default=1)),
            ('status', self.gf('django.db.models.fields.PositiveIntegerField')(default=1)),
        ))
        db.send_create_signal('activitylog', ['Backlink'])

        # Adding unique constraint on 'Backlink', fields ['site', 'url', 'referrer']
        db.create_unique('activitylog_backlink', ['site_id', 'url', 'referrer'])

        # Adding model 'ProfileIP'
        db.create_table('activitylog_profileip', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('modified', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('cache_version', self.gf('django.db.models.fields.PositiveIntegerField')(default=0)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('profile', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['{ACTIVITYLOG_PROFILE_MODEL}'.format(ACTIVITYLOG_PROFILE_MODEL=ACTIVITYLOG_PROFILE_MODEL)])),
            ('ip', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['activitylog.IP'])),
        ))
        db.send_create_signal('activitylog', ['ProfileIP'])

        # Adding unique constraint on 'ProfileIP', fields ['ip', 'user']
        db.create_unique('activitylog_profileip', ['ip_id', 'user_id'])

        # Adding unique constraint on 'ProfileIP', fields ['ip', 'profile']
        db.create_unique('activitylog_profileip', ['ip_id', 'profile_id'])

        # Adding model 'ProfileUserAgent'
        db.create_table('activitylog_profileuseragent', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('modified', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('cache_version', self.gf('django.db.models.fields.PositiveIntegerField')(default=0)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('profile', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['{ACTIVITYLOG_PROFILE_MODEL}'.format(ACTIVITYLOG_PROFILE_MODEL=ACTIVITYLOG_PROFILE_MODEL)])),
            ('agent', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['activitylog.UserAgent'])),
        ))
        db.send_create_signal('activitylog', ['ProfileUserAgent'])

        # Adding unique constraint on 'ProfileUserAgent', fields ['agent', 'user']
        db.create_unique('activitylog_profileuseragent', ['agent_id', 'user_id'])

        # Adding unique constraint on 'ProfileUserAgent', fields ['agent', 'profile']
        db.create_unique('activitylog_profileuseragent', ['agent_id', 'profile_id'])


    def backwards(self, orm):
        # Removing unique constraint on 'ProfileUserAgent', fields ['agent', 'profile']
        db.delete_unique('activitylog_profileuseragent', ['agent_id', 'profile_id'])

        # Removing unique constraint on 'ProfileUserAgent', fields ['agent', 'user']
        db.delete_unique('activitylog_profileuseragent', ['agent_id', 'user_id'])

        # Removing unique constraint on 'ProfileIP', fields ['ip', 'profile']
        db.delete_unique('activitylog_profileip', ['ip_id', 'profile_id'])

        # Removing unique constraint on 'ProfileIP', fields ['ip', 'user']
        db.delete_unique('activitylog_profileip', ['ip_id', 'user_id'])

        # Removing unique constraint on 'Backlink', fields ['site', 'url', 'referrer']
        db.delete_unique('activitylog_backlink', ['site_id', 'url', 'referrer'])

        # Deleting model 'UserAgent'
        db.delete_table('activitylog_useragent')

        # Deleting model 'IP'
        db.delete_table('activitylog_ip')

        # Deleting model 'Backlink'
        db.delete_table('activitylog_backlink')

        # Deleting model 'ProfileIP'
        db.delete_table('activitylog_profileip')

        # Deleting model 'ProfileUserAgent'
        db.delete_table('activitylog_profileuseragent')


    models = {
        apm_key: freeze_apps(apm_app)[apm_key],
        'activitylog.backlink': {
            'Meta': {'unique_together': "((u'site', u'url', u'referrer'),)", 'object_name': 'Backlink'},
            'cache_version': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'referrer': ('django.db.models.fields.URLField', [], {'max_length': '100'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'status': ('django.db.models.fields.PositiveIntegerField', [], {'default': '1'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '100'}),
            'visits': ('django.db.models.fields.PositiveIntegerField', [], {'default': '1'})
        },
        'activitylog.ip': {
            'Meta': {'object_name': 'IP'},
            'address': ('django.db.models.fields.IPAddressField', [], {'null': 'True', 'default': 'None', 'max_length': '15', 'blank': 'True', 'unique': 'True', 'db_index': 'True'}),
            'cache_version': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'hostname': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'number': ('django.db.models.fields.BigIntegerField', [], {'default': 'None', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'profiles': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['{ACTIVITYLOG_PROFILE_MODEL}']".format(ACTIVITYLOG_PROFILE_MODEL=ACTIVITYLOG_PROFILE_MODEL), 'through': "orm['activitylog.ProfileIP']", 'symmetrical': 'False'})
        },
        'activitylog.profileip': {
            'Meta': {'unique_together': "((u'ip', u'user'), (u'ip', u'profile'))", 'object_name': 'ProfileIP'},
            'cache_version': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ip': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['activitylog.IP']"}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'profile': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['{ACTIVITYLOG_PROFILE_MODEL}']".format(ACTIVITYLOG_PROFILE_MODEL=ACTIVITYLOG_PROFILE_MODEL)}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'activitylog.profileuseragent': {
            'Meta': {'unique_together': "((u'agent', u'user'), (u'agent', u'profile'))", 'object_name': 'ProfileUserAgent'},
            'agent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['activitylog.UserAgent']"}),
            'cache_version': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'profile': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['{ACTIVITYLOG_PROFILE_MODEL}']".format(ACTIVITYLOG_PROFILE_MODEL=ACTIVITYLOG_PROFILE_MODEL)}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'activitylog.useragent': {
            'Meta': {'object_name': 'UserAgent'},
            'cache_version': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255', 'db_index': 'True'}),
            'profiles': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['{ACTIVITYLOG_PROFILE_MODEL}']".format(ACTIVITYLOG_PROFILE_MODEL=ACTIVITYLOG_PROFILE_MODEL), 'through': "orm['activitylog.ProfileUserAgent']", 'symmetrical': 'False'})
        },
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['activitylog']



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2012 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""Test for common routines and models."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from django.conf import settings
from django.test import TestCase
from django.utils.unittest import skipUnless


@skipUnless("lck.dummy.defaults" in settings.INSTALLED_APPS,
            "Requires the lck.dummy.defaults to be installed.")
class TestModels(TestCase):
    def test_creation(self):
        from django.contrib.auth.models import User
        from lck.dummy.defaults.models import Profile

        u = User(username='pinky', email='pinky@brain.com',
                 first_name='Pinky', last_name='Mouse')
        with self.assertRaises(Profile.DoesNotExist):
            u.get_profile()
        u.save()
        self.assertEqual(u.username, u.get_profile().nick)

    def test_profile_attribute_passing(self):
        self.test_creation()
        from lck.dummy.defaults.models import Profile
        p = Profile.objects.all()[0]
        with self.assertNumQueries(1):
            self.assertEqual(p.nick, 'pinky')
            self.assertEqual(p.email, 'pinky@brain.com')
            self.assertEqual(p.get_full_name(), 'Pinky Mouse')
            self.assertFalse(p.is_anonymous())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2011 by Łukasz Langa

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from datetime import datetime
from time import time

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.core.cache import cache
from django.core.exceptions import ImproperlyConfigured
from django.db import models as db
from django.db import transaction

try:
    from django.utils.timezone import now
except ImportError:
    now = datetime.now

from lck.django.activitylog.models import UserAgent, IP, ProfileIP,\
    ProfileUserAgent, Backlink, ACTIVITYLOG_PROFILE_MODEL
from lck.django.common import model_is_user, remote_addr

class OptionBag(object): pass
_backlink_url_max_length = Backlink._meta.get_field_by_name(
    'url')[0].max_length
_backlink_referrer_max_length = Backlink._meta.get_field_by_name(
    'referrer')[0].max_length
BACKLINKS_LOCAL_SITES = getattr(
    settings, 'BACKLINKS_LOCAL_SITES', 'current',
)
CURRENTLY_ONLINE_INTERVAL = getattr(
    settings, 'CURRENTLY_ONLINE_INTERVAL', 120,
)
ACTIVITYLOG_MODE = getattr(
    settings, 'ACTIVITYLOG_MODE', 'sync',
)
ACTIVITYLOG_TASK_EXPIRATION = getattr(
    settings, 'ACTIVITYLOG_TASK_EXPIRATION', 30,
)
if ACTIVITYLOG_MODE == 'sync':
    def maybe_async(function):
        result = OptionBag()
        result.delay = function
        return result
    serial_execution = False
elif ACTIVITYLOG_MODE == 'rq':
    from django_rq import job
    maybe_async = job(
        'activitylog',
        timeout=ACTIVITYLOG_TASK_EXPIRATION,
        result_ttl=ACTIVITYLOG_TASK_EXPIRATION,
    )
    serial_execution = True
elif ACTIVITYLOG_MODE == 'celery':
    import celery
    maybe_async = celery.task(
        expires=ACTIVITYLOG_TASK_EXPIRATION,
    )
    serial_execution = True


@maybe_async
@transaction.commit_on_success
def update_activity(user_id, address, agent, _now_dt):
    ip, _ = IP.concurrent_get_or_create(
        address=address, fast_mode=serial_execution,
    )
    if agent:
        agent, _ = UserAgent.concurrent_get_or_create(
            name=agent, fast_mode=serial_execution,
        )
    else:
        agent = None
    if user_id:
        profile = User.objects.get(pk=user_id)
        if not model_is_user(ACTIVITYLOG_PROFILE_MODEL):
            profile = profile.get_profile()
        last_active = profile.last_active
        if not last_active or CURRENTLY_ONLINE_INTERVAL <= 3 * (_now_dt -
                last_active).seconds:
            # we're not using save() to bypass signals etc.
            profile.__class__.objects.filter(pk=profile.pk).update(
                last_active=_now_dt)
        pip, _ = ProfileIP.concurrent_get_or_create(
            ip=ip, profile=profile, fast_mode=serial_execution,
        )
        ProfileIP.objects.filter(pk=pip.pk).update(modified=_now_dt)
        if agent:
            pua, _ = ProfileUserAgent.concurrent_get_or_create(
                agent=agent, profile=profile, fast_mode=serial_execution,
            )
            ProfileUserAgent.objects.filter(pk=pua.pk).update(
                modified=_now_dt,
            )
    return ip, agent


@maybe_async
@transaction.commit_on_success
def update_backlinks(path_info, referrer, current_site):
    backlink, backlink_created = Backlink.concurrent_get_or_create(
        site=current_site,
        url=path_info[:_backlink_url_max_length],
        referrer=referrer[:_backlink_referrer_max_length],
        fast_mode=serial_execution,
    )
    if not backlink_created:
        # we're not using save() to bypass signals etc.
        Backlink.objects.filter(id=backlink.id).update(
            modified=now(), visits=db.F('visits') + 1,
        )


class ActivityMiddleware(object):
    """Updates the `last_active` profile field for every logged in user with
    the current timestamp. It pragmatically stores a new value every 40 seconds
    (one third of the seconds specified ``CURRENTLY_ONLINE_INTERVAL`` setting).
    """

    def process_request(self, request):
        _now_dt = now()
        _now_ts = int(time())
        delta = _now_ts - CURRENTLY_ONLINE_INTERVAL
        users_online = cache.get('users_online', {})
        guests_online = cache.get('guests_online', {})
        users_touched = False
        guests_touched = False
        if request.user.is_authenticated():
            users_online[request.user.id] = _now_ts
            users_touched = True
            user_id = request.user.id
        else:
            guest_sid = request.COOKIES.get(settings.SESSION_COOKIE_NAME)
            if guest_sid:
                guests_online[guest_sid] = _now_ts
                guests_touched = True
            user_id = None
        for uid in users_online.keys():
            try:
                is_stale_key = users_online[uid] < delta
            except TypeError:
                is_stale_key = True
            if is_stale_key:
                users_touched = True
                del users_online[uid]
        if users_touched:
            cache.set('users_online', users_online, 60 * 60 * 24)
        for gid in guests_online.keys():
            try:
                is_stale_key = guests_online[gid] < delta
            except TypeError:
                is_stale_key = True
            if is_stale_key:
                guests_touched = True
                del guests_online[gid]
        if guests_touched:
            cache.set('guests_online', guests_online, 60 * 60 * 24)
        address = remote_addr(request)
        agent = request.META.get('HTTP_USER_AGENT')
        request.activity = update_activity.delay(
            user_id, address, agent, _now_dt,
        )

    if BACKLINKS_LOCAL_SITES == 'current':
        def process_response(self, request, response):
            current_site = Site.objects.get(id=settings.SITE_ID)
            try:
                ref = request.META.get('HTTP_REFERER', '').split('//')[1]
                if response.status_code // 100 == 2 and not \
                    (ref == current_site.domain or ref.startswith(
                        current_site.domain + '/')):
                    update_backlinks.delay(
                        request.META['PATH_INFO'],
                        request.META['HTTP_REFERER'],
                        current_site,
                    )
            except (IndexError, UnicodeDecodeError):
                pass
            return response
    elif BACKLINKS_LOCAL_SITES == 'all':
        def process_response(self, request, response):
            try:
                ref = request.META.get('HTTP_REFERER', '').split('//')[1]
                if response.status_code // 100 == 2:
                    for site in Site.objects.all():
                        if ref == site.domain or \
                                ref.startswith(site.domain + '/'):
                            break
                        if site.id == settings.SITE_ID:
                            current_site = site
                    else:
                        update_backlinks.delay(
                            request.META['PATH_INFO'],
                            request.META['HTTP_REFERER'],
                            current_site,
                        )
            except (IndexError, UnicodeDecodeError):
                pass
            return response
    else:
        raise ImproperlyConfigured(
            "Unsupported value for BACKLINKS_LOCAL_SITES: {!r}"
            "".format(BACKLINKS_LOCAL_SITES),
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2012 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""Test for common routines and models."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from django.conf import settings
from django.test import TestCase
from django.utils.unittest import skipUnless


class TestHelpers(TestCase):
    def test_cut(self):
        from lck.django.common import cut
        self.assertEqual(cut("123456789", length=9), "123456789")
        self.assertEqual(cut("123456789ABC", length=9), "123456789 (...)")
        self.assertEqual(cut("123456789ABC", length=9, trailing=" ..."), "123456789 ...")
        self.assertEqual(cut("123456789ABC", length=12, trailing=" ..."), "123456789ABC")
        self.assertEqual(cut(None, length=12, trailing=" ..."), None)
        self.assertEqual(cut("", length=12, trailing=" ..."), "")
        self.assertEqual(cut("", length=0, trailing=" ..."), "")
        self.assertEqual(cut("123456789", length=-1), "12345678 (...)")


@skipUnless("lck.dummy.defaults" in settings.INSTALLED_APPS,
            "Requires lck.dummy.defaults to be installed.")
class TestModels(TestCase):
    def test_dirty_fields(self):
        from lck.dummy.defaults.models import TimeConscious
        tc1 = TimeConscious(name='tc1')
        last_modified = tc1.modified
        self.assertFalse(tc1.dirty_fields, "object initialized")
        self.assertFalse(tc1.cache_version)
        tc1.save()
        self.assertEqual(tc1.modified, last_modified)
        self.assertFalse(tc1.dirty_fields, "object saved")
        tc1.cache_version = 23
        self.assertIn('cache_version', tc1.dirty_fields,
                      "insignificant fields")
        tc1.name = 'TC1'
        self.assertIn('name', tc1.dirty_fields, "significant fields")
        tc1.save()
        self.assertNotEqual(tc1.modified, last_modified)
        last_modified = tc1.modified
        self.assertFalse(tc1.dirty_fields, "object saved")
        tc1.mark_dirty('name')
        self.assertIn('name', tc1.dirty_fields, "forced dirty")
        self.assertEqual(tc1.dirty_fields['name'], tc1.name)
        tc1.save()
        self.assertNotEqual(tc1.modified, last_modified)
        last_modified = tc1.modified
        self.assertFalse(tc1.dirty_fields, "object saved")
        tc1.mark_dirty('name')
        self.assertIn('name', tc1.dirty_fields, "forced dirty")
        self.assertEqual(tc1.dirty_fields['name'], tc1.name)
        tc1.mark_clean('name')
        self.assertFalse(tc1.dirty_fields, "cleaned forced dirty")
        tc1.save()
        self.assertEqual(tc1.modified, last_modified)
        self.assertFalse(tc1.dirty_fields, "object saved")
        tc1.name = 'tc1'
        self.assertIn('name', tc1.dirty_fields, "significant fields")
        tc1.mark_clean('name')
        self.assertIn('name', tc1.dirty_fields, "clean on a changed field")
        tc1.mark_clean('name', force=True)
        self.assertNotIn('name', tc1.dirty_fields,
                         "forced clean on a changed field")
        tc1.save()
        self.assertEqual(tc1.modified, last_modified)
        tc2 = TimeConscious.objects.get(pk=tc1.pk)
        self.assertEqual(tc2.name, 'tc1', "saved anyway")


    def test_concurrent_get_or_create(self):
        from lck.dummy.defaults.models import CurrentlyConcurrent
        existing = CurrentlyConcurrent(
            name='existing',
            field1=0,
            field2=0,
            field3=0,
            field4=0,
            field5=0,
            field6=0,
        )
        existing.save()
        self.assertEqual(existing.id, 1)
        # use all unique fields properly and separate others to `defaults`
        existing2, created = CurrentlyConcurrent.concurrent_get_or_create(
            name='existing',
            field1=0,
            field2=0,
            field3=0,
            field4=0,
            defaults=dict(
                field5=0,
                field6=0,
            ),
        )
        self.assertFalse(created)
        self.assertEqual(existing, existing2)
        # try to throw non-unique fields to the arguments list
        with self.assertRaises(AssertionError):
            existing2, created = CurrentlyConcurrent.concurrent_get_or_create(
                name='existing',
                field1=0,
                field2=0,
                field3=0,
                field4=0,
                field5=0,
                field6=0,
            )
        # try to not use all unique fields in the arguments list
        with self.assertRaises(AssertionError):
            existing2, created = CurrentlyConcurrent.concurrent_get_or_create(
                name='existing',
                field1=0,
                field4=0,
            )



# -*- coding: utf-8 -*-
### NOTE:
### This snippet will be executed in the ``settings.py`` context and is not an
### actual Python module that you would import in a traditional sense. When you
### do import its name, you get the path to this file so you can ``execfile()``
### it.

import django.core.management
import django.utils.translation
import django.utils.translation.trans_real


def django_core_management__find_management_module(app_name):
    """
    Determines the path to the management module for the given app_name,
    without actually importing the application or the management module.

    Raises ImportError if the management module cannot be found for any reason.

    Supports namespace packages. Based on Django 1.4.2.
    """
    import os.path

    parts = app_name.split('.')
    parts.append('management')
    parts = [str(p) for p in parts]

    management_module = __import__(app_name + '.management', fromlist=parts)
    path = management_module.__file__

    if path.endswith(('__init__.py', '__init__.pyc')):
        path = os.path.split(path)[0]

    return path


class Trans(object):
    """
    The purpose of this class is to store the actual translation function upon
    receiving the first call to that function. After this is done, changes to
    USE_I18N will have no effect to which function is served upon request. If
    your tests rely on changing USE_I18N, you can delete all the functions
    from _trans.__dict__.

    Note that storing the function with setattr will have a noticeable
    performance effect, as access to the function goes the normal path,
    instead of using __getattr__.

    Supports namespace packages. Based on Django 1.4.2.
    """

    def __getattr__(self, real_name):
        from os import path
        import warnings
        from django.conf import settings
        from django.utils.importlib import import_module
        if settings.USE_I18N:
            from django.utils.translation import trans_real as trans
            # Make sure the project's locale dir isn't in LOCALE_PATHS
            if settings.SETTINGS_MODULE is not None:
                parts = settings.SETTINGS_MODULE.split('.')
                for index in xrange(1, len(parts)):
                    project = import_module(".".join(parts[0:index]))
                    if hasattr(project, '__file__'):
                        break
                else:
                    project = import_module("django")
                project_locale_path = path.normpath(
                    path.join(path.dirname(project.__file__), 'locale'))
                normalized_locale_paths = [path.normpath(locale_path)
                    for locale_path in settings.LOCALE_PATHS]
                if (path.isdir(project_locale_path) and
                        not project_locale_path in normalized_locale_paths):
                    warnings.warn("Translations in the project directory "
                                  "aren't supported anymore. Use the "
                                  "LOCALE_PATHS setting instead.",
                                  DeprecationWarning)
        else:
            from django.utils.translation import trans_null as trans
        setattr(self, real_name, getattr(trans, real_name))
        return getattr(trans, real_name)


class Translations(dict):
    """Overrides the default dictionary of translations so that the broken
    built-in implementation of django.utils.translation.trans_real.translation()
    doesn't need to be invoked.

    Supports namespace packages. Based on Django 1.4.2.
    """

    def get(self, language, default=None):
        """django.utils.translation.trans_real.translation() with a few changes."""
        import gettext
        import os.path
        import sys
        from django.utils.importlib import import_module

        t = dict.get(self, language)
        if t:
            return t

        from django.conf import settings

        globalpath = os.path.join(os.path.dirname(sys.modules[settings.__module__].__file__), 'locale')

        if settings.SETTINGS_MODULE is not None:
            parts = settings.SETTINGS_MODULE.split('.')
            for index in xrange(1, len(parts)):
                project = import_module(".".join(parts[0:index]))
                if hasattr(project, '__file__'):
                    break
            else:
                project = import_module("django")
            projectpath = os.path.join(os.path.dirname(project.__file__), 'locale')
        else:
            projectpath = None

        def _fetch(lang, fallback=None):

            global _translations

            res = dict.get(self, lang, None)
            if res is not None:
                return res

            loc = django.utils.translation.trans_real.to_locale(lang)

            def _translation(path):
                try:
                    t = gettext.translation('django', path, [loc], django.utils.translation.trans_real.DjangoTranslation)
                    t.set_language(lang)
                    return t
                except IOError:
                    return None

            res = _translation(globalpath)

            # We want to ensure that, for example,  "en-gb" and "en-us" don't share
            # the same translation object (thus, merging en-us with a local update
            # doesn't affect en-gb), even though they will both use the core "en"
            # translation. So we have to subvert Python's internal gettext caching.
            base_lang = lambda x: x.split('-', 1)[0]
            if base_lang(lang) in [base_lang(trans) for trans in dict.keys(self)]:
                res._info = res._info.copy()
                res._catalog = res._catalog.copy()

            def _merge(path):
                t = _translation(path)
                if t is not None:
                    if res is None:
                        return t
                    else:
                        res.merge(t)
                return res

            for appname in reversed(settings.INSTALLED_APPS):
                app = import_module(appname)
                apppath = os.path.join(os.path.dirname(app.__file__), 'locale')

                if os.path.isdir(apppath):
                    res = _merge(apppath)

            localepaths = [os.path.normpath(path) for path in settings.LOCALE_PATHS]
            if (projectpath and os.path.isdir(projectpath) and
                    os.path.normpath(projectpath) not in localepaths):
                res = _merge(projectpath)

            for localepath in reversed(settings.LOCALE_PATHS):
                if os.path.isdir(localepath):
                    res = _merge(localepath)

            if res is None:
                if fallback is not None:
                    res = fallback
                else:
                    return gettext.NullTranslations()
            self[lang] = res
            return res

        default_translation = _fetch(settings.LANGUAGE_CODE)
        current_translation = _fetch(language, fallback=default_translation)

        return current_translation


# PATCH: a more generic find_management_module that supports namespace
# packages.
# COMPATIBILITY: Django 1.1.0 - 1.4.0
django.core.management.find_management_module = \
    django_core_management__find_management_module

# PATCH: a more generic _trans object that supports namespace
# packages.
# COMPATIBILITY: Django 1.3.0 - 1.4.2
django.utils.translation._trans = Trans()

# PATCH: django.utils.translation.trans_real.translation() uses globals and is
# generally tricky to patch. We patch the data structure used for holding
# translations instead so that the translation() method successfully gets
# an object out it and returns early (before raising an exception due to its
# lack of support for namespace packages).
# COMPATIBILITY: Django 1.3.0 - 1.4.2
django.utils.translation.trans_real._translations = Translations()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2013 by Łukasz Langa
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from datetime import datetime
import sys

from django.core.management.base import BaseCommand
from django.contrib.auth.models import User
from django.contrib.sessions.models import Session


class Command(BaseCommand):
    help = ("Returns the username and e-mail given a session ID.")

    requires_model_validation = False

    def handle(self, *args, **options):
        session_key = None
        for session_key in args:
            try:
                s = Session.objects.get(session_key=session_key)
            except Session.DoesNotExist:
                print("error: Session {} not found.".format(session_key),
                      file=sys.stderr)
            else:
                d = s.get_decoded()
                print("{session.session_key}\n"
                        "expires {session.expire_date}".format(
                            session=s
                        )
                )
                if '_auth_user_id' in d:
                    user = User.objects.get(pk=d['_auth_user_id'])
                    print("userid={user.id}\n"
                          "username={user.username}\nemail={user.email}\n"
                          "".format(user=user)
                    )
                else:
                    print("anonymous\n")
        if session_key is None:
            now = datetime.now()
            for s in Session.objects.filter(expire_date__gte=now).order_by(
                    '-expire_date'):
                print(s.get_decoded())
